@@ -1161,7 +1161,7 @@ None by default.
 
 =head1 AUTHOR
 
-Sean Egan, E<lt>seanegan@bigfoot.comE<gt>
+Sean Egan, E<lt>seanegan:bigfoot_comE<gt>
 
 =head1 SEE ALSO
 
